package DDLSSoftware;

import java.util.Timer;
import java.util.TimerTask;

import DDLSSoftware.DDLSViewIF.INPUT;

public class DDLSController {

	public enum PW_STATE{
		UNSET, SET_1, SET_2, SET_3, SET,
		INPUT_1, INPUT_2, INPUT_3, RIGHT,
		W_1, W_2, W_3, WRONG,
		CANCEL;
	}
	public enum LOCK_STATE{
		OPEN, CLOSE;
	}
	public enum DOOR_STATE{
		OPEN, CLOSE;
	}
	
	private CommandIF command = null;
	
	private PW_STATE passwordState;
	private LOCK_STATE lockState;
	private DOOR_STATE doorState;
	
	private INPUT[] password = new INPUT[4];
	
	Timer passwordTimer = null;
	Timer doorTimer = null;
	
	public DDLSController(CommandIF command){
		this.command = command;
		this.passwordState = PW_STATE.UNSET;
		this.lockState = LOCK_STATE.CLOSE;
		this.doorState = DOOR_STATE.CLOSE;
		for(int i = 0; i<4;i++){
			password[i] = null;
		}

	}
	
	public void getInput(INPUT input){
		switch(input){
		case INPUT_0:
		case INPUT_1:
		case INPUT_2:
		case INPUT_3:
		case INPUT_4:
		case INPUT_5:
		case INPUT_6:
		case INPUT_7:
		case INPUT_8:
		case INPUT_9:
			passwordController(input);
			break;
		case INPUT_LOCK_BUTTON:
		case INPUT_KEY_SENSOR:
		case INPUT_DOOR_SENSOR:
			lockController(input);
			break;
		case INPUT_COVER_SENSOR:
				if(this.passwordState == PW_STATE.SET_1 ||
				this.passwordState == PW_STATE.SET_2 ||
				this.passwordState == PW_STATE.SET_3){
					this.passwordState = PW_STATE.UNSET;
				}
				if(this.passwordState == PW_STATE.INPUT_1||
					this.passwordState == PW_STATE.INPUT_2||
					this.passwordState == PW_STATE.INPUT_3){
					this.passwordState = PW_STATE.SET;
				}
			break;
		}
		System.out.println("Password State = " + passwordState);
		System.out.println("lock State = " + lockState);
	}
	public void passwordController(INPUT input){
		switch(this.passwordState){
		case UNSET:
		case SET_1:
		case SET_2:
		case SET_3:
			setPassword(input);
			break;
		case SET:
		case INPUT_1:
		case INPUT_2:
		case INPUT_3:
		case W_1:
		case W_2:
		case W_3:
			comparePassword(input);
			break;
		default:
			break;
		}
	}
	public void setPassword(INPUT input){
		switch(this.passwordState){
		case UNSET:
			this.password[0] = input;
			this.passwordState = PW_STATE.SET_1;
			break;
		case SET_1:
			this.password[1] = input;
			this.passwordState = PW_STATE.SET_2;
			break;
		case SET_2:
			this.password[2] = input;
			this.passwordState = PW_STATE.SET_3;
			break;
		case SET_3:
			this.password[3] = input;
			this.passwordState = PW_STATE.SET;
			break;
		default:
			break;
		}
	}
	private void comparePassword(INPUT input) {
		switch(this.passwordState){
		case SET:
			if(this.password[0] == input){
				this.passwordState = PW_STATE.INPUT_1;
			}
			else{
				this.passwordState = PW_STATE.W_1;
			}
			break;
		case INPUT_1:
			if(this.password[1] == input){
				this.passwordState = PW_STATE.INPUT_2;
			}
			else{
				this.passwordState = PW_STATE.W_2;
			}
			break;
		case INPUT_2:
			if(this.password[2] == input){
				this.passwordState = PW_STATE.INPUT_3;
			}
			else{
				this.passwordState = PW_STATE.W_3;
			}
			break;
		case INPUT_3:
			if(this.password[3] == input){
				this.passwordState = PW_STATE.RIGHT;
				lockController(input);
			}
			else{
				this.passwordState = PW_STATE.WRONG;
			}
			break;
		case W_1:
			this.passwordState = PW_STATE.W_2;
			break;
		case W_2:
			this.passwordState = PW_STATE.W_3;
			break;
		case W_3:
			this.passwordState = PW_STATE.WRONG;
			break;
		default:
			break;
		}
		AlarmController();
		backlightController();
		if(passwordTimer != null){
			passwordTimer.cancel();
			passwordTimer = null;
		}
		passwordTimer = new Timer();
		passwordTimer.schedule(new TimerTask() {
			
			@Override
			public void run() {
				// TODO Auto-generated method stub
				cancelPassword();
			}
		}, 5000);
	}
	void cancelPassword(){
		passwordState = PW_STATE.CANCEL;
		AlarmController();
	}
	public void AlarmController(){
		switch(this.passwordState){
		case SET_1:
		case SET_2:
		case SET_3:
			break;
		case INPUT_1:
		case INPUT_2:
		case INPUT_3:
		case W_1:
		case W_2:
		case W_3:
			command.turnOnAlarm1();
			break;
		case RIGHT:
			command.turnOnAlarm1();
			this.passwordState = PW_STATE.SET;
			break;
		case CANCEL:
			command.turnOnAlarm2();
			this.passwordState = PW_STATE.SET;
			break;
		case WRONG:
			command.turnOnAlarm3();
			this.passwordState = PW_STATE.SET;
			break;
		default:
			break;
		}
	}
	public void backlightController(){
		command.turnOnBacklight();
	}
	public void lockController(INPUT input){
		if(this.passwordState == PW_STATE.RIGHT){
			openLock();
			return;
		}
		switch(input){
		case INPUT_KEY_SENSOR:
			openLock();
			break;
		case INPUT_LOCK_BUTTON:
			if(lockState == LOCK_STATE.CLOSE){
				openLock();
			}
			else{
				closeLock();
			}
			break;
		case INPUT_DOOR_SENSOR:
			if(doorState == DOOR_STATE.OPEN){
				DoorClosed();
			}
			else{
				DoorOpened();
			}
			break;
		default:
			break;
		}
		
	}
	public void openLock(){
		command.openLockDevice();
		if(doorTimer != null){
			doorTimer.cancel();
			doorTimer = null;
		}
		if(doorState == DOOR_STATE.CLOSE){
			DoorClosed();
		}
		lockState = LOCK_STATE.OPEN;
	}
	public void closeLock(){
		command.closeLockDevice();
		if(doorTimer != null){
			doorTimer.cancel();
			doorTimer = null;
		}
		lockState = LOCK_STATE.CLOSE;
	}
	public void DoorOpened(){
		doorState = DOOR_STATE.OPEN;
		if(doorTimer != null){
			doorTimer.cancel();
			doorTimer = null;
		}
	}
	public void DoorClosed(){
		doorState = DOOR_STATE.CLOSE;
		if(doorTimer != null){
			doorTimer.cancel();
			doorTimer = null;
		}
		doorTimer = new Timer();
		doorTimer.schedule(new TimerTask() {
			
			@Override
			public void run() {
				// TODO Auto-generated method stub
				closeLock();
			}
		}, 3000);
		
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
}
